#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "stuff.h"

int getIntValue(GtkWidget *widget, char *widgetname)
{
   GtkWidget *target =lookup_widget (GTK_WIDGET (widget), widgetname);
   
   return gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (target));
}


char *getStringValue(GtkWidget *widget, char *widgetname)
{
   GtkWidget *target =lookup_widget (GTK_WIDGET (widget), widgetname);
   
   return  gtk_entry_get_text (GTK_ENTRY (target));
}

Params *getParams(GtkWidget *widget)
{
   static Params params;
   
   params.blocksize = 4; /* DEFAULT size per set of images */

   params.nrperindex = getIntValue(widget,"nrperindex");
   params.thumbwidth = getIntValue(widget,"thumbwidth");
   params.startnr    = getIntValue(widget,"startnr");
   params.endnr      = getIntValue(widget,"endnr");
   
   params.basename      = getStringValue(widget,"basename");
   params.fullsizedir   = getStringValue(widget,"fullsize");
   params.thumbnailsdir = getStringValue(widget,"thumbnails");
   params.jpgextension  = getStringValue(widget,"jpgextension");

   return &params;
}


void showParams(Params *params)
{
   printf("Blocksize       : %d \n",params->blocksize);
   
   printf("Nr. per index   : %d \n",params->nrperindex);
   printf("Thumbwidth      : %d \n",params->thumbwidth);
   printf("Nr. first photo : %d \n",params->startnr);
   printf("Nr. last photo  : %d \n",params->endnr);
   
   printf("Basename        : %s \n",params->basename);
   printf("Fullsize   path : %s \n",params->fullsizedir);
   printf("Thumbnails path : %s \n",params->thumbnailsdir);
   printf("Jpeg extension  : %s \n",params->jpgextension);
}


void makeBlock(FILE *output, Params *params)
{
   int blockcount=1;
   char name[256];
   char fname[1024];
   char tname[1024];
   struct stat buf;
   
   fprintf(stderr, "Making new block ...\n");

   fprintf(output, "<TABLE>\n");
   
   while(blockcount <= params->blocksize)
   {
      snprintf(name,  sizeof(name)  -1, "%s%04d",params->basename,
	       params->phcount);
      snprintf(fname, sizeof(fname) -1, "%s/%s.%s",params->fullsizedir,
	       name,params->jpgextension);
      snprintf(tname, sizeof(tname) -1, "%s/%s-%d.%s",params->thumbnailsdir,
	       name,params->thumbwidth,params->jpgextension);
      
      if(stat(fname,&buf) == 0) // only if photo really exists
      {
	 fprintf(stderr," Now adding %s ...\n",fname);
	 
	 if(blockcount%2) /* odd */
	   fprintf(output,"  <TR>\n");
	 
	 fprintf(output,
		 "    <TD>\n"
		 "      <A HREF=%s> <IMG SRC=%s> <br> %04d </A>\n"
		 "    </TD>\n",
		 fname,tname,params->phcount);
	 
	 if(! blockcount%2) /* even */
	   fprintf(output,"  </TR>\n");

	 blockcount++;
      }
      
      params->phcount++;
      if(params->phcount > params->endnr)
	break;
   }
      
   fprintf(output, "</TABLE>\n");
}


void makeOneRow(FILE *output,Params *params)
{
   fprintf(stderr,"Making new row ...\n");   
					
   // 1 row = 2 blocks = 2 x blocksize photos (currently 8)
   fprintf(output,
	   "  <TR>\n"
	   "    <TD>\n"
	   "      <UL>\n");
   
   
   makeBlock(output,params);
   fprintf(output,
           "      </UL>\n"
	   "    </TD>\n"
	   "    <TD>\n"
           "      <UL>\n");
   
   makeBlock(output,params);
   fprintf(output,
           "      </UL>\n"
	   "    </TD>\n"
	   "  </TR>\n");
}


void makeTopIndex(Params *params,int indexcount)
{
   int indexnr=0;
   char outputname[256];
   char title[100];
   FILE *output=stdout; /* stdout only for debugging */
   struct stat buf;
   time_t t;

   snprintf(outputname,sizeof(outputname) -1,"%s_topindex.html",
	    params->basename);
   
   snprintf(title,     sizeof(title) -1,
	    "Master index for photos with base %s", params->basename);
   
#ifndef DEBUG      
   output = fopen(outputname,"w");
   if(output == NULL)
   {
      fprintf(stderr,"Can't open %s, exiting ...\n",outputname);
      exit(1); // give warning
   }
#endif
   
   fprintf(output,
	   "<HTML>\n"
	   "<HEAD>\n"
	   "<TITLE> %s </TITLE>\n"
	   "</HEAD>\n\n"
	   "<BODY BGCOLOR='lightblue' TEXT=#FF0000 VLINK=#3000F0>\n"
	   "<CENTER>\n"
	   "<H1> %s </H1>\n"
	   "<P>\n"
	   "<HR>\n"
	   "<UL>\n"
	   , title, title);
   
   while(indexnr < indexcount)
   {
      indexnr++;
      snprintf(outputname,sizeof(outputname) -1,"%s_indexnr_%d.html",
	       params->basename,indexnr);
      
      if(stat(outputname,&buf) == 0) // only if photo really exists
	fprintf(output,
		"    <LI>\n"
		"      <A HREF=%s> Index with number %d </A>\n"
		"    </LI>\n",
		outputname,indexnr);
   }
   
   time(&t);
   fprintf(output,
	   "  </UL>\n"
	   "</CENTER>\n"
	   "<HR>\n"
	   "Created by photo2html (GTK/Linux; KL 2003) : %s\n"
	   "</BODY>\n"
	   "</HTML>\n"
	   ,ctime(&t));
   
#ifndef DEBUG
   fclose(output);
#endif
}


void makeButtons(FILE *output, Params *params, int indexnr, int phcount)
{
   fprintf(output,
	   "<P>\n"
	   "<TABLE> <TR>\n"
	   "<TR>");
   
   if(indexnr > 1)
     fprintf(output,
	     "  <TD>\n"
	     "    <A HREF=%s_indexnr_%d.html> [PREV] </A>\n"
	     "  </TD>\n",
	     params->basename, indexnr - 1 );
   
   fprintf(output,
	   "  <TD>\n"
	   "    <A HREF=%s_topindex.html> [TOP] </A>\n"
	   "  </TD>\n",
	   params->basename);
   
   /* NOTE: photo counter is not yet increased at the top button row,
    * so we need to adjust it for the top row ! */
   
   if(phcount < params->endnr)
     fprintf(output,
	     "  <TD>\n"
	     "    <A HREF=%s_indexnr_%d.html> [NEXT] </A>\n"
	     "  </TD>\n",
	     params->basename, indexnr + 1 );
   
   fprintf(output,
	   "</TR> </TABLE>\n"
	   "<P>\n"
	   "<HR>\n");
}


void makeHeader(FILE *output, Params *params,int indexnr)
{
   char title[100];

   snprintf(title, sizeof(title) - 1,
	    "Index %d: %d photos starting at photo %04d",
	    indexnr,params->nrperindex,params->startnr);
   
   fprintf(output,
	   "<HTML>\n"
	   "<HEAD>\n"
	   "<TITLE> %s </TITLE>\n"
	   "</HEAD>\n"
	   "\n"
	   "<BODY BGCOLOR='lightblue' TEXT=#FF0000 VLINK=#3000F0>"
	   "\n"
	   "<center>\n"
	   "<h1> %s </h1>\n"
	   "<p>\n"
	   "</center>\n"
	   "<HR>\n"
	   , title, title);
   
   makeButtons(output, params, indexnr, params->phcount + params->nrperindex);
}

void generateHtml(GtkButton *button, gpointer user_data)
{
   Params *params;
   int indexnr=1;
   int row;
   int endrow;
   char outputname[256];
   FILE *output=stdout; /* stdout only for debugging */
   struct stat buf;
   
   params = getParams((GtkWidget *)button);
   params->phcount=params->startnr -1;
   showParams(params);
   
   /* Check if thumbnails exist ! */
   if(stat(params->thumbnailsdir,&buf) != 0)
     createDialogbox("Thumbnails directory doesn't exist\n"
		     "Create and generate thumbnails ?",
		     NULL,generateThumbnails);
   
   /* NOTE: Currently nrperindex should be a multiple of 2 x blocksize ! */
   while(params->phcount <= params->endnr)
   {
      snprintf(outputname,sizeof(outputname) -1,"%s_indexnr_%d.html",
	       params->basename,indexnr);
      
      fprintf(stderr, "Now writing to %s ....\n",outputname);
      
#ifndef DEBUG      
      output = fopen(outputname,"w");
      if(output == NULL)
      {
	 fprintf(stderr,"Can't open %s, exiting ...\n",outputname);
	 exit(1); // give warning
      }
#endif
      
      makeHeader(output, params, indexnr);
      fprintf(output, "<TABLE>\n");
      
      row    = 1;    
      endrow = params->nrperindex / (2 * params->blocksize);
      while( row <= endrow )
      {
	 makeOneRow(output,params); // creates 2 blocks
	 row++;
      }

      fprintf(output,"</TABLE>\n" "<HR>\n");
      makeButtons(output, params, indexnr, params->phcount);
      fprintf(output,"</BODY>\n" "</HTML>\n");
      
      params->startnr=params->phcount;
      fprintf(stderr, "photo counter : %d\n",params->phcount);
      
      indexnr++;
      
#ifndef DEBUG
      fclose(output);
#endif
   }
   
   makeTopIndex(params,indexnr);
}

void viewHtml(GtkButton *button, gpointer user_data)
{
   Params *params;
   char command[1024];
   
   params = getParams((GtkWidget *)button);
   
   snprintf(command,sizeof(command) -1,
	    BROWSER " file:///$PWD/%s_topindex.html &",
	    params->basename);
   system(command);
}

